/*
 *
 * Copyright (C) 2000  Robert Olsson, Jens Ls 
 * Swedish University of Agricultural Sciences
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include "pim_common.h"

char b1[16], b2[16], b3[16], b4[16];  /* For inet_2a */

struct interface *
get_iflist_ifp(int idx)
{
  listnode node;
  struct interface *ifp;

  for (node = listhead (iflist); node; nextnode (node)) {
      ifp = getdata (node);
      if(ifp->ifindex == idx) return ifp;
    }
  return NULL;
}

struct pim_interface *
pim_if_new (struct interface *ifp)
{
  struct pim_interface *pi;

  pi = XMALLOC (MTYPE_PIM_IF, sizeof (struct pim_interface));
  bzero (pi, sizeof (struct pim_interface));

  /* Set zebra interface pointer. */
  pi->ifp = ifp;

  return pi;
}

/* pimd's interface node. */
struct cmd_node interface_node =
{
  INTERFACE_NODE,
  "%s(config-if)# ",
};

void
pim_if_free (struct pim_interface *pi)
{
  XFREE (MTYPE_PIM_IF, pi);
}

int
pim_if_new_hook (struct interface *ifp)
{
  ifp->info = pim_if_new (ifp);
   return 0;
}

int
pim_if_delete_hook (struct interface *ifp)
{
  pim_if_free ((struct pim_interface *) ifp->info);

  return 0;
}

int
interface_config_write (struct vty *vty)
{
  listnode node, n1;
  struct interface *ifp;
  struct pim_interface *pi;
  int write = 0;

  for (node = listhead (iflist); node; nextnode (node))
    {
      ifp = getdata (node);
      pi = ifp->info;

#if 0
      if (!if_is_up (ifp)) continue;
#endif
      vty_out (vty, "!%s", VTY_NEWLINE);
      vty_out (vty, "interface %s%s", ifp->name,
	       VTY_NEWLINE);
      
      write++; /* Check usage write ROHACK */

      if(pi->mode == PIM_DENSE_MODE) 
	vty_out(vty, " ip pim dense-mode%s", VTY_NEWLINE);
      else if(pi->mode == PIM_DENSE_SLOW_START_MODE)
	vty_out(vty, " ip pim dense-slow-start-mode%s", VTY_NEWLINE);
      else if(pi->mode == PIM_SPARSE_MODE) 
	vty_out(vty, " ip pim sparse-mode%s", VTY_NEWLINE);
      else if(pi->mode == PIM_SPARSE_DENSE_MODE) 
	vty_out(vty, " ip pim sparse-dense-mode%s", VTY_NEWLINE);


      if(pi->version == PIM_VERSION_1 )
		vty_out(vty, " ip pim version 1%s", VTY_NEWLINE);

      if(pi->version == PIM_VERSION_2 )
		vty_out(vty, " ip pim version 2%s", VTY_NEWLINE);

      if(pi->hello_period_set) {
	vty_out(vty, " ip pim hello-interval %d %s", 
		pi->hello_period, VTY_NEWLINE);
      }


      if(pi->flags & IF_PREFERENCE_SET)
	vty_out(vty, " ip pim preference  %u%s", 
		pi->preference, VTY_NEWLINE);

      if(pi->flags & IF_METRIC_SET)
	vty_out(vty, " ip pim metric  %u%s", 
		pi->metric, VTY_NEWLINE);

      if( pi->igmp_version_conf == 1) 
	vty_out(vty, " ip igmp version 1 %s", VTY_NEWLINE);

      if(pi->igmp_static_group_list) {
	for (n1 = listhead (pi->igmp_static_group_list); n1; nextnode (n1))
	  {
	    u_int32_t *g  = (u_int32_t *) getdata (n1);
	    vty_out(vty, " ip igmp join-group %s%s", inet_2a(*g, b1), 
		    VTY_NEWLINE);
	  }
      }  
    }
  return write;
}

DEFUN (if_ip_pim,
       if_ip_pim_cmd,
       "ip pim (dense-mode|dense-slow-start-mode|sparse-mode|sparse-dense-mode)",
       "PIM interface commands\n"
       "PIM mode\n"
       "Specify PIM dense-mode\n"
       "Specify PIM dense-slow-start-mode\n"
       "Specify PIM sparse-mode\n"
       "Specify PIM sparse-dense-mode\n")
{
  struct interface *ifp;
  struct pim_interface *pi;

  ifp = vty->index;
  pi = ifp->info;

  pi->mode =  PIM_NO;

  if (strncmp (argv[0], "sparse-dense", 11) == 0)
    pi->mode = PIM_SPARSE_DENSE_MODE;
  else if (strncmp (argv[0], "dense-mode", 9) == 0)
    pi->mode = PIM_DENSE_MODE;
  else if (strncmp (argv[0], "dense-slow-start-mode", 9) == 0)
    pi->mode = PIM_DENSE_SLOW_START_MODE;
  else if (strncmp (argv[0], "sparse", 5) == 0)
    pi->mode = PIM_SPARSE_MODE;

   pim_if_start(ifp);

  return CMD_SUCCESS;
}

DEFUN (if_no_ip_pim,
       if_no_ip_pim_cmd,
       "no ip pim",
       NO_STR
       "PIM interface commands\n"
       "disable pim")
{
  struct interface *ifp;
  struct pim_interface *pi;

  ifp = vty->index;
  pi = ifp->info;

  pi->mode = PIM_NO;
  pim_if_stop(ifp);

  return CMD_SUCCESS;
}

DEFUN (if_ip_pim_version,
       if_ip_pim_version_cmd,
       "ip pim version (1|2)",
       NO_STR
       "PIM interface commands\n"
       "Specify PIM version\n"
       "Specify PIM version 1\n"
       "Specify PIM version 2\n")
{
  struct interface *ifp;
  struct pim_interface *pi;

  ifp = vty->index;
  pi = ifp->info;

  if (strncmp (argv[0], "1", 1) == 0)
    pi->version = PIM_VERSION_1;
  else if (strncmp (argv[0], "2", 1) == 0)
    pi->version = PIM_VERSION_2;

  return CMD_SUCCESS;
}

DEFUN (if_no_ip_pim_version,
       if_no_ip_pim_version_cmd,
       "no ip pim version",
       NO_STR
       "PIM version\n"
       "Specify PIM version\n")
{
  struct interface *ifp;
  struct pim_interface *pi;

  ifp = vty->index;
  pi = ifp->info;

  pi->version = PIM_VERSION_NONE;

  return CMD_SUCCESS;
}

DEFUN (if_ip_pim_hello_interval,
       if_ip_pim_hello_interval_cmd,
       "ip pim hello-interval <0-65535>",
       "PIM interface commands\n"
       "Specify PIM hello_interval\n"
       "Specify PIM hello_interval\n")
{
  struct interface *ifp;
  struct pim_interface *pi;
  u_int16_t i;

  ifp = vty->index;
  pi = ifp->info;
  
  i = strtol (argv[0], NULL, 10);

  pi->hello_period = i;
  pi->hello_period_set = 1;

  if(pi->t_hello) {
    thread_cancel(pi->t_hello);
    pi->t_hello = thread_add_timer(master, hello_thread, pi, 0);
  }

  return CMD_SUCCESS;
}

DEFUN (if_no_ip_pim_hello_interval,
       if_no_ip_pim_hello_interval_cmd,
       "no ip pim hello-interval <0-65535>",
       "PIM interface commands\n"
       "Unset PIM hello_interval\n"
       "Unset PIM hello_interval\n")
{
  struct interface *ifp;
  struct pim_interface *pi;

  ifp = vty->index;
  pi = ifp->info;
  
  pi->hello_period = HELLO_INTERVAL; /* Default */
  pi->hello_period_set = 0;

  if(pi->t_hello) {
    thread_cancel(pi->t_hello);
    pi->t_hello = thread_add_timer(master, hello_thread, pi, 0);
  }

  return CMD_SUCCESS;
}


void
vifs_init ()
{
  listnode node;
  struct interface *ifp;
  struct pim_interface *pi;

  for (node = listhead (iflist); node; nextnode (node))
    {
      ifp = getdata (node);
      pi = ifp->info;
  
#if 0
      if (!if_is_up (ifp)) continue;
#endif
    }
}

void
pim_interface_init()
{

  if_init(); /* Init iflist */
  if_add_hook (IF_NEW_HOOK, pim_if_new_hook);
  if_add_hook (IF_DELETE_HOOK, pim_if_delete_hook);
  
  /* Install interface node. */
  install_node (&interface_node, interface_config_write);
  install_element (CONFIG_NODE, &interface_cmd);
  
  install_element (INTERFACE_NODE, &if_ip_pim_cmd);
  install_element (INTERFACE_NODE, &if_no_ip_pim_cmd);
  install_element (INTERFACE_NODE, &if_ip_pim_version_cmd);
  install_element (INTERFACE_NODE, &if_no_ip_pim_version_cmd);

  install_element (INTERFACE_NODE, &if_ip_pim_hello_interval_cmd);
  install_element (INTERFACE_NODE, &if_no_ip_pim_hello_interval_cmd);


  install_element (INTERFACE_NODE, &config_end_cmd);
  install_element (INTERFACE_NODE, &config_exit_cmd);
  install_element (INTERFACE_NODE, &config_help_cmd);
}
